==> airbridge_flutter_sdk-1.0.0.tar.gz: ios/Classes/AFLEventAPI.m <==
//
//  AFLEventAPI.m
//  airbridge_flutter_sdk
//
//  Created by WOF on 08/10/2019.
//

#import "AFLEventAPI.h"

// Airbridge
#import "ABInAppEvent.h"
#import "ABEcommerceEvent.h"
#import "ABUserEvent.h"
// AB
#import "ABGet.h"

@implementation AFLEventAPI

//
// flutter plugin setting
//

+ (void) registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"airbridge_flutter_sdk/method/event"
                                                                binaryMessenger:registrar.messenger];
    AFLEventAPI* instance = [[AFLEventAPI alloc] init];
    
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void) handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSDictionary* arguments = [ABGet type:NSDictionary.class object:call.arguments];
    
    if ([@"sendSignIn" isEqualToString:call.method]) {
        [self sendSignIn:arguments result:result];
    } else if ([@"sendSignUp" isEqualToString:call.method]) {
        [self sendSignUp:arguments result:result];
    } else if ([@"sendSignOut" isEqualToString:call.method]) {
        [self sendSignOut:arguments result:result];
    } else if ([@"sendViewHome" isEqualToString:call.method]) {
        [self sendViewHome:arguments result:result];
    } else if ([@"sendViewSearchResult" isEqualToString:call.method]) {
        [self sendViewSearchResult:arguments result:result];
    } else if ([@"sendViewProductList" isEqualToString:call.method]) {
        [self sendViewProductList:arguments result:result];
    } else if ([@"sendViewProductDetail" isEqualToString:call.method]) {
        [self sendViewProductDetail:arguments result:result];
    } else if ([@"sendAddToCart" isEqualToString:call.method]) {
        [self sendAddToCart:arguments result:result];
    } else if ([@"sendPurchase" isEqualToString:call.method]) {
        [self sendPurchase:arguments result:result];
    } else if ([@"sendCustom" isEqualToString:call.method]) {
        [self sendCustom:arguments result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

//
// flutter method
//

- (void) sendSignIn:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* user = [ABGet type:NSDictionary.class dictionary:arguments key:@"user"];
    NSDictionary* option = [ABGet type:NSDictionary.class dictionary:arguments key:@"option"];

    ABUser* nativeUser = convertUser(user);

    ABUserEvent* event = [[ABUserEvent alloc] initWithUser:nativeUser];

    addOptionToEvent(event, option);

    [event sendSignin];
}

- (void) sendSignUp:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* user = [ABGet type:NSDictionary.class dictionary:arguments key:@"user"];
    NSDictionary* option = [ABGet type:NSDictionary.class dictionary:arguments key:@"option"];

    ABUser* nativeUser = convertUser(user);

    ABUserEvent* event = [[ABUserEvent alloc] initWithUser:nativeUser];

    addOptionToEvent(event, option);

    [event sendSignup];
}

- (void) sendSignOut:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [ABGet type:NSDictionary.class dictionary:arguments key:@"option"];

    ABUserEvent* event = [[ABUserEvent alloc] init];

    addOptionToEvent(event, option);

    [event expireUser];
}

- (void) sendViewHome:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [ABGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] init];

    addOptionToEvent(event, option);
    
    [event sendViewHome];
    
    result(nil);
}

- (void) sendViewSearchResult:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [ABGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    NSString* query = [ABGet type:NSString.class dictionary:arguments key:@"query"];
    NSArray* products = [ABGet type:NSArray.class dictionary:arguments key:@"products"];
    NSArray<ABProduct*>* nativeProducts = convertProducts(products);
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] initWithProducts:nativeProducts];

    addOptionToEvent(event, option);
    event.query = query;
    
    [event sendViewSearchResult];
    
    result(nil);
}

- (void) sendViewProductList:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [ABGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    NSString* listID = [ABGet type:NSString.class dictionary:arguments key:@"listID"];
    NSArray* products = [ABGet type:NSArray.class dictionary:arguments key:@"products"];
    NSArray<ABProduct*>* nativeProducts = convertProducts(products);
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] initWithProducts:nativeProducts];

    addOptionToEvent(event, option);
    event.productListID = listID;
    
    [event sendViewProductList];
    
    result(nil);
}

- (void) sendViewProductDetail:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [ABGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    NSArray* products = [ABGet type:NSArray.class dictionary:arguments key:@"products"];
    NSArray<ABProduct*>* nativeProducts = convertProducts(products);
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] initWithProducts:nativeProducts];

    addOptionToEvent(event, option);
    
    [event sendViewProductDetail];
    
    result(nil);
}

- (void) sendAddToCart:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [ABGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    NSString* cartID = [ABGet type:NSString.class dictionary:arguments key:@"cartID"];
    NSString* currency = [ABGet type:NSString.class dictionary:arguments key:@"currency"];
    NSNumber* total = [ABGet type:NSNumber.class dictionary:arguments key:@"total"];
    NSArray* products = [ABGet type:NSArray.class dictionary:arguments key:@"products"];
    NSArray<ABProduct*>* nativeProducts = convertProducts(products);
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] initWithProducts:nativeProducts];

    addOptionToEvent(event, option);
    event.cartID = cartID;
    event.currency = currency;
    event.eventValue = total;
    
    [event sendAddProductToCart];
    
    result(nil);
}

- (void) sendPurchase:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [ABGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    NSString* transactionID = [ABGet type:NSString.class dictionary:arguments key:@"transactionID"];
    NSNumber* isInAppPurchase = [ABGet type:NSNumber.class dictionary:arguments key:@"isInAppPurchase"];
    NSString* currency = [ABGet type:NSString.class dictionary:arguments key:@"currency"];
    NSNumber* total = [ABGet type:NSNumber.class dictionary:arguments key:@"total"];
    NSArray* products = [ABGet type:NSArray.class dictionary:arguments key:@"products"];
    NSArray<ABProduct*>* nativeProducts = convertProducts(products);
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] initWithProducts:nativeProducts];

    addOptionToEvent(event, option);
    event.transactionID = transactionID;
    event.isInAppPurchase = isInAppPurchase.boolValue;
    event.currency = currency;
    event.eventValue = total;
    
    [event sendCompleteOrder];
    
    result(nil);
}

- (void) sendCustom:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSString* category = [ABGet type:NSString.class dictionary:arguments key:@"category"];
    NSDictionary* option = [ABGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    ABInAppEvent* event = [[ABInAppEvent alloc] init];

    addOptionToEvent(event, option);
    [event setCategory:category];
    
    [event send];
}

//
// private - tool
//

/**
 * Add option infomation to event
 * @discussion  add nil when some data has unmatch type
 * @param       event
 *                  target event
 * @param       option
 *                  dictionary which has option infomation
 */
static void addOptionToEvent(ABInAppEvent* event, NSDictionary* __nullable option) {
    if (option == nil) return;
    
    NSString* action = [ABGet type:NSString.class dictionary:option key:@"action"];
    NSString* label = [ABGet type:NSString.class dictionary:option key:@"label"];
    NSNumber* value = [ABGet type:NSNumber.class dictionary:option key:@"value"];
    NSDictionary* attributes = [ABGet type:NSDictionary.class dictionary:option key:@"attributes"];
    
    [event setAction:action];
    [event setLabel:label];
    [event setValue:value];
    [event setCustoms:attributes];
}

/**
 * Convert dictionary to ABUser
 * @discussion  return nil when user is nil
 *
 *              add nil when some data has unmatch type
 * @param       user
 *                  dictionary which has user infomation
 * @return      ABUser
 */
static ABUser* __nullable convertUser(NSDictionary* __nullable user) {
    if(user == nil) return nil;
    
    NSString* ID = [ABGet type:NSString.class dictionary:user key:@"id"];
    NSString* email = [ABGet type:NSString.class dictionary:user key:@"email"];
    NSString* phone = [ABGet type:NSString.class dictionary:user key:@"phone"];
    
    ABUser* nativeUser = [[ABUser alloc] init];
    nativeUser.ID = ID;
    nativeUser.email = email;
    nativeUser.phone = phone;
    
    return nativeUser;
}

/**
 * Convert array-of-dictionarys to array-of-ABProducts
 * @discussion  return nil when products is nil
 *
 *              add nil when some data has unmatch type
 * @param       products
 *                  array of dictionarys
 * @return      array of ABProducts
 */
static NSArray<ABProduct*>* __nullable convertProducts(NSArray* __nullable products) {
    if (products == nil) return nil;
    
    NSMutableArray<ABProduct*>* nativeProducts = [[NSMutableArray alloc] init];
    
    for (int i =0; i<products.count; i++){
        ABProduct* nativeProduct = convertProduct(products[i]);
        [nativeProducts addObject:nativeProduct];
    }
    
    return nativeProducts;
}

/**
 * Convert dictionary to ABProduct
 * @discussion  return nil when product is nil
 *
 *              add nil when some data has unmatch type
 * @param       product
 *                  dictionary which has product infomation
 * @return      ABProduct
 */
static ABProduct* __nullable convertProduct(NSDictionary* __nullable product) {
    if (product == nil) return nil;

    NSString* ID = [ABGet type:NSString.class dictionary:product key:@"id"];
    NSString* name = [ABGet type:NSString.class dictionary:product key:@"name"];
    NSString* currency = [ABGet type:NSString.class dictionary:product key:@"currency"];
    NSNumber* quantity = [ABGet type:NSNumber.class dictionary:product key:@"quantity"];
    NSNumber* price = [ABGet type:NSNumber.class dictionary:product key:@"price"];
    NSNumber* position = [ABGet type:NSNumber.class dictionary:product key:@"position"];
    
    ABProduct* nativeProduct = [[ABProduct alloc] init];
    nativeProduct.idx = ID;
    nativeProduct.name = name;
    nativeProduct.currency = currency;
    nativeProduct.quantity = quantity;
    nativeProduct.price = price;
    nativeProduct.orderPosition = position;
    
    return nativeProduct;
}

@end

==> airbridge_flutter_sdk-1.1.0.tar.gz: ios/Classes/AFLEventAPI.m <==
//
//  AFLEventAPI.m
//  airbridge_flutter_sdk
//
//  Created by WOF on 08/10/2019.
//

#import "AFLEventAPI.h"

// Airbridge
#import "ABInAppEvent.h"
#import "ABEcommerceEvent.h"
#import "ABUserEvent.h"
// AB
#import "AFLGet.h"

@implementation AFLEventAPI

//
// flutter plugin setting
//

+ (void) registerWithRegistrar:(NSObject<FlutterPluginRegistrar>*)registrar {
    FlutterMethodChannel* channel = [FlutterMethodChannel methodChannelWithName:@"airbridge_flutter_sdk/method/event"
                                                                binaryMessenger:registrar.messenger];
    AFLEventAPI* instance = [[AFLEventAPI alloc] init];
    
    [registrar addMethodCallDelegate:instance channel:channel];
}

- (void) handleMethodCall:(FlutterMethodCall*)call result:(FlutterResult)result {
    NSDictionary* arguments = [AFLGet type:NSDictionary.class object:call.arguments];
    
    if ([@"sendSignIn" isEqualToString:call.method]) {
        [self sendSignIn:arguments result:result];
    } else if ([@"sendSignUp" isEqualToString:call.method]) {
        [self sendSignUp:arguments result:result];
    } else if ([@"sendSignOut" isEqualToString:call.method]) {
        [self sendSignOut:arguments result:result];
    } else if ([@"sendViewHome" isEqualToString:call.method]) {
        [self sendViewHome:arguments result:result];
    } else if ([@"sendViewSearchResult" isEqualToString:call.method]) {
        [self sendViewSearchResult:arguments result:result];
    } else if ([@"sendViewProductList" isEqualToString:call.method]) {
        [self sendViewProductList:arguments result:result];
    } else if ([@"sendViewProductDetail" isEqualToString:call.method]) {
        [self sendViewProductDetail:arguments result:result];
    } else if ([@"sendAddToCart" isEqualToString:call.method]) {
        [self sendAddToCart:arguments result:result];
    } else if ([@"sendPurchase" isEqualToString:call.method]) {
        [self sendPurchase:arguments result:result];
    } else if ([@"sendCustom" isEqualToString:call.method]) {
        [self sendCustom:arguments result:result];
    } else {
        result(FlutterMethodNotImplemented);
    }
}

//
// flutter method
//

- (void) sendSignIn:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* user = [AFLGet type:NSDictionary.class dictionary:arguments key:@"user"];
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];

    ABUser* nativeUser = convertUser(user);

    ABUserEvent* event = [[ABUserEvent alloc] initWithUser:nativeUser];

    addOptionToEvent(event, option);

    [event sendSignin];
}

- (void) sendSignUp:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* user = [AFLGet type:NSDictionary.class dictionary:arguments key:@"user"];
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];

    ABUser* nativeUser = convertUser(user);

    ABUserEvent* event = [[ABUserEvent alloc] initWithUser:nativeUser];

    addOptionToEvent(event, option);

    [event sendSignup];
}

- (void) sendSignOut:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];

    ABUserEvent* event = [[ABUserEvent alloc] init];

    addOptionToEvent(event, option);

    [event expireUser];
}

- (void) sendViewHome:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] init];

    addOptionToEvent(event, option);
    
    [event sendViewHome];
    
    result(nil);
}

- (void) sendViewSearchResult:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    NSString* query = [AFLGet type:NSString.class dictionary:arguments key:@"query"];
    NSArray* products = [AFLGet type:NSArray.class dictionary:arguments key:@"products"];
    NSArray<ABProduct*>* nativeProducts = convertProducts(products);
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] initWithProducts:nativeProducts];

    addOptionToEvent(event, option);
    event.query = query;
    
    [event sendViewSearchResult];
    
    result(nil);
}

- (void) sendViewProductList:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    NSString* listID = [AFLGet type:NSString.class dictionary:arguments key:@"listID"];
    NSArray* products = [AFLGet type:NSArray.class dictionary:arguments key:@"products"];
    NSArray<ABProduct*>* nativeProducts = convertProducts(products);
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] initWithProducts:nativeProducts];

    addOptionToEvent(event, option);
    event.productListID = listID;
    
    [event sendViewProductList];
    
    result(nil);
}

- (void) sendViewProductDetail:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    NSArray* products = [AFLGet type:NSArray.class dictionary:arguments key:@"products"];
    NSArray<ABProduct*>* nativeProducts = convertProducts(products);
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] initWithProducts:nativeProducts];

    addOptionToEvent(event, option);
    
    [event sendViewProductDetail];
    
    result(nil);
}

- (void) sendAddToCart:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    NSString* cartID = [AFLGet type:NSString.class dictionary:arguments key:@"cartID"];
    NSString* currency = [AFLGet type:NSString.class dictionary:arguments key:@"currency"];
    NSNumber* total = [AFLGet type:NSNumber.class dictionary:arguments key:@"total"];
    NSArray* products = [AFLGet type:NSArray.class dictionary:arguments key:@"products"];
    NSArray<ABProduct*>* nativeProducts = convertProducts(products);
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] initWithProducts:nativeProducts];

    addOptionToEvent(event, option);
    event.cartID = cartID;
    event.currency = currency;
    event.eventValue = total;
    
    [event sendAddProductToCart];
    
    result(nil);
}

- (void) sendPurchase:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    NSString* transactionID = [AFLGet type:NSString.class dictionary:arguments key:@"transactionID"];
    NSNumber* isInAppPurchase = [AFLGet type:NSNumber.class dictionary:arguments key:@"isInAppPurchase"];
    NSString* currency = [AFLGet type:NSString.class dictionary:arguments key:@"currency"];
    NSNumber* total = [AFLGet type:NSNumber.class dictionary:arguments key:@"total"];
    NSArray* products = [AFLGet type:NSArray.class dictionary:arguments key:@"products"];
    NSArray<ABProduct*>* nativeProducts = convertProducts(products);
    
    ABEcommerceEvent* event = [[ABEcommerceEvent alloc] initWithProducts:nativeProducts];

    addOptionToEvent(event, option);
    event.transactionID = transactionID;
    event.isInAppPurchase = isInAppPurchase.boolValue;
    event.currency = currency;
    event.eventValue = total;
    
    [event sendCompleteOrder];
    
    result(nil);
}

- (void) sendCustom:(nullable NSDictionary*)arguments result:(FlutterResult)result {
    NSString* category = [AFLGet type:NSString.class dictionary:arguments key:@"category"];
    NSDictionary* option = [AFLGet type:NSDictionary.class dictionary:arguments key:@"option"];
    
    ABInAppEvent* event = [[ABInAppEvent alloc] init];

    addOptionToEvent(event, option);
    [event setCategory:category];
    
    [event send];
}

//
// private - tool
//

/**
 * Add option infomation to event
 * @discussion  add nil when some data has unmatch type
 * @param       event
 *                  target event
 * @param       option
 *                  dictionary which has option infomation
 */
static void addOptionToEvent(ABInAppEvent* event, NSDictionary* __nullable option) {
    if (option == nil) return;
    
    NSString* action = [AFLGet type:NSString.class dictionary:option key:@"action"];
    NSString* label = [AFLGet type:NSString.class dictionary:option key:@"label"];
    NSNumber* value = [AFLGet type:NSNumber.class dictionary:option key:@"value"];
    NSDictionary* attributes = [AFLGet type:NSDictionary.class dictionary:option key:@"attributes"];
    
    [event setAction:action];
    [event setLabel:label];
    [event setValue:value];
    [event setCustoms:attributes];
}

/**
 * Convert dictionary to ABUser
 * @discussion  return nil when user is nil
 *
 *              add nil when some data has unmatch type
 * @param       user
 *                  dictionary which has user infomation
 * @return      ABUser
 */
static ABUser* __nullable convertUser(NSDictionary* __nullable user) {
    if(user == nil) return nil;
    
    NSString* ID = [AFLGet type:NSString.class dictionary:user key:@"id"];
    NSString* email = [AFLGet type:NSString.class dictionary:user key:@"email"];
    NSString* phone = [AFLGet type:NSString.class dictionary:user key:@"phone"];
    NSDictionary<NSString*, NSString*>* alias = [AFLGet type:NSDictionary.class dictionary:user key:@"alias"];
    NSDictionary<NSString*, NSObject*>* attributes = [AFLGet type:NSDictionary.class dictionary:user key:@"attributes"];
    
    ABUser* nativeUser = [[ABUser alloc] init];
    nativeUser.ID = ID;
    nativeUser.email = email;
    nativeUser.phone = phone;
    nativeUser.alias = alias;
    nativeUser.attributes = attributes;
    
    return nativeUser;
}

/**
 * Convert array-of-dictionarys to array-of-ABProducts
 * @discussion  return nil when products is nil
 *
 *              add nil when some data has unmatch type
 * @param       products
 *                  array of dictionarys
 * @return      array of ABProducts
 */
static NSArray<ABProduct*>* __nullable convertProducts(NSArray* __nullable products) {
    if (products == nil) return nil;
    
    NSMutableArray<ABProduct*>* nativeProducts = [[NSMutableArray alloc] init];
    
    for (int i =0; i<products.count; i++){
        ABProduct* nativeProduct = convertProduct(products[i]);
        [nativeProducts addObject:nativeProduct];
    }
    
    return nativeProducts;
}

/**
 * Convert dictionary to ABProduct
 * @discussion  return nil when product is nil
 *
 *              add nil when some data has unmatch type
 * @param       product
 *                  dictionary which has product infomation
 * @return      ABProduct
 */
static ABProduct* __nullable convertProduct(NSDictionary* __nullable product) {
    if (product == nil) return nil;

    NSString* ID = [AFLGet type:NSString.class dictionary:product key:@"id"];
    NSString* name = [AFLGet type:NSString.class dictionary:product key:@"name"];
    NSString* currency = [AFLGet type:NSString.class dictionary:product key:@"currency"];
    NSNumber* quantity = [AFLGet type:NSNumber.class dictionary:product key:@"quantity"];
    NSNumber* price = [AFLGet type:NSNumber.class dictionary:product key:@"price"];
    NSNumber* position = [AFLGet type:NSNumber.class dictionary:product key:@"position"];
    
    ABProduct* nativeProduct = [[ABProduct alloc] init];
    nativeProduct.idx = ID;
    nativeProduct.name = name;
    nativeProduct.currency = currency;
    nativeProduct.quantity = quantity;
    nativeProduct.price = price;
    nativeProduct.orderPosition = position;
    
    return nativeProduct;
}

@end
